      subroutine maelseg_mtra(imtra,ierror)

      implicit none

************************************************************************
*     Author     : Peter J. Weyers <weyers@ifh.de>
*     using code by Rainer Mankel 
*     Description: fills RSEG Table rows with information retrieved from the
*     MC for MTRA row number imtra simulating the muon detector
*     reconstruction to be used as a seed for the upstream
*     RANGER algorithm
      INTEGER imtra             ! in    MTRA row number
      INTEGER ierror            ! out   0=success
                                !       1=track skipped because it wasn't a muon
                                !       2=No usable MIMP found
                                !       3=Problems with MMBOOK()
*     
************************************************************************

#include "arte/Arte.fi"
#include "auto/MIMP.fi"
#include "auto/MTRA.fi"
#include "auto/MTRX.fi"
#include "auto/RSEG.fi"
#include "auto/TOPP.fi"
#include "match98.fi"
#include "matarx.fi"
#include "rang/zdomain.fi"
#include "pris/zmpack.fi"
#include "rang/zf.fi"

      integer irseg_new,i,j
      INTEGER imimp,indx,imimp_tmp
      INTEGER old_nrseg,sgn,rseg_ecal
      REAL z,E,dE,x(5),dx(5),mc_q,mc_p
      REAL xyz(3),dxyz(3)
      REAL z_ecal ! Value of Z where to emulate ECAL hits

      PARAMETER ( z_ecal = 1450 )


#include "arte/Arte.ff"
#include "auto/MIMP.ff"
#include "auto/MTRA.ff"
#include "auto/MTRX.ff"
#include "auto/RSEG.ff"
#include "auto/TOPP.ff"


      ierror=100
      IF(OMTRA_TOPP(imtra).NE.code_eplus.AND.
     $     OMTRA_TOPP(imtra).NE.code_eminus) THEN
         ierror=1
         IF(ma_verblvl.GE.10) WRITE(*,*)
     $        ' maelseg_mtra: MC track ',imtra,' is not an electron',
     $        ' - skipped'
         RETURN
      ENDIF

      IF(ma_verblvl.GE.10)
     $     WRITE(*,*) " maelseg_mtra: Searchin MIMP for lepton",
     $     imtra
      imimp=0
c     locate last mimp
      indx=0
      DO WHILE(maslcmimp_mtra(imimp_tmp,indx,imtra)) 
         if(imimp.EQ.0) imimp=imimp_tmp
         if(fmimp_z(imimp).LT.fmimp_z(imimp_tmp).AND.
     $        fmimp_z(imimp_tmp).LT.z_ecal)
     $        imimp=imimp_tmp
      ENDDO


      IF(imimp.EQ.0.OR.fmimp_z(imimp).lt.(z_ecal-150)) THEN
         IF(ma_verblvl.GT.4)
     $      WRITE(*,*) ' maelseg_mtra: no usable MIMP found - skipping!'
         ierror=2
         RETURN
      ELSE 
         IF(ma_verblvl.GT.8) WRITE(*,*) 
     $        ' maelseg_mtra: found MIMP',imimp,'z=',fmimp_z(imimp)

         IF(maecalswitch(xyz,dxyz,E,dE,imtra,imimp)) THEN
            DO sgn=-1,1,2
c            DO sgn=1,1
               mc_q=ITOPP_CHRG(OMTRA_TOPP(imtra))/3.

               IF(ma_verblvl.GE.8) WRITE(*,*) ' maelseg_mtra:',
     $              ' sign=',sgn,' charge=',mc_q
               
               E=ABS(E)*sgn

               x(1)=xyz(1)
               dx(1)=dxyz(1)
               x(2)=xyz(2)
               dx(2)=dxyz(2)
               z=xyz(3)
               
               x(3)=x(1)/z
               dx(3)=1
               x(4)=X(2)/z
               dx(4)=1

               CALL maecalseg(z,x,dx,E,dE,imtra,rseg_ecal)
               
c               IQ(IQRSEG_CMP(rseg_ecal))=RSEGc_ecal
               
c     With RSEG_CMP != 0, matxfit will just update the tangents,
c     but not extend the segment to the target.

               CALL matxfit98(rseg_ecal,ma_tar,ma_dtar,6,ierror)

               IF(ierror.NE.0)THEN
                  ma_txfit_errors=ma_txfit_errors+1
               ENDIF

               IF(ma_verblvl.GE.8) THEN
                  WRITE(*,*) ' maelseg_mtra: z=',z
                  IF(imimp.NE.0) THEN
                     WRITE(*,*) ' maelseg_mtra: p fit/mc(mimp) p=',
     $                 1/FRSEG_PF(rseg_ecal),mc_p/mc_q
                     WRITE(*,*) ' maelseg_mtra: tx/ty (mimp)=',
     $                 FMIMP_TX(imimp),FMIMP_TY(imimp)
                     WRITE(*,*) ' maelseg_mtra: tx/ty (fit)=',
     $                 FRSEG_TXF(rseg_ecal),FRSEG_TYF(rseg_ecal)
                  ELSE
                     WRITE(*,*) ' maelseg_mtra: p fit/mc(mtra) p=',
     $                    1/FRSEG_PF(rseg_ecal),mc_p/mc_q
                  ENDIF
               ENDIF
            ENDDO               ! sign
         ENDIF                  ! maecalswitch()

      ENDIF

      RETURN
      END






