      INTEGER FUNCTION MAGETGTAR(imtra,error)
      
      IMPLICIT NONE

************************************************************************
* Author     : Peter J. Weyers <weyers@ifh.de>
*
* Description: Returns the GTAR row number of the wire "closest" to the 
*              primary vertex of the given MC track
*              Returns 0 in case of error
* Arguments:

      INTEGER imtra   ! in   Particle to be considered as coming from target
      INTEGER error   ! out  = 0 : success                          

************************************************************************

#include "arte/Arte.fi"
#include "auto/GTAR.fi"
#include "auto/MTRA.fi"
#include "auto/MTRX.fi"
#include "auto/MVER.fi"
c #include "pris/zpstat.fi"
#define MAGETGTAR_FUNC_DEF
#include "match98.fi"

      INTEGER ibver,JPsi_ibver
c      INTEGER pstat_i
      INTEGER gtar_ir
      REAL    ver_x,ver_y ! Koordinates of the MC primary B0 vertex
      REAL    ver_z
c      REAL    tar_x,tar_y,tar_z
      REAL    min_dist_sq,dist_sq
      CHARACTER*4 ori_str
c       REAL    mu_x,mu_y         ! mu MC track parameters of extrapolation
c                           ! to target z value
      REAL    mx,my,bx,by
#include "arte/Arte.ff"
#include "auto/GTAR.ff"
#include "auto/MTRA.ff"
#include "auto/MTRX.ff"
#include "auto/MVER.ff"

************************************************************************

      IF(ma_verblvl.GE.20)
     $     WRITE(*,*) ' magetgtar: started'

      error=100 ! To be changed before returning

      IF(NGTAR().LT.1) THEN
         IF(ma_verblvl.GE.2)
     $        WRITE (*,*) ' magetgtar: GTAR table is empty !!!,'
         magetgtar=0
         error=2
         GOTO 999 ! Return
      ENDIF
      

      IF(imtra.LE.0.OR.imtra.GT.NMTRA()) THEN
         magetgtar=0
         error=3
         IF(ma_verblvl.GE.4)
     $        WRITE(*,*) ' magetgtar: Invalid MTRA row number:',imtra
         GOTO 999 ! Return
      ENDIF

      ibver=OMTRX_BVER(imtra)

      ver_x=FMVER_X(ibver)
      ver_y=FMVER_Y(ibver)
      ver_z=FMVER_Z(ibver)
   
      magetgtar=0
      
      min_dist_sq=1000.

      DO gtar_ir=1,NGTAR()
         IF(ABS(FGTAR_X(gtar_ir)).LT.
     $      ABS(FGTAR_y(gtar_ir))) THEN ! horizontal wire
            dist_sq=(ver_y-FGTAR_Y(gtar_ir))**2+
     $              (ver_z-FGTAR_Z(gtar_ir))**2
            ori_str='(h)'
         ELSE                   ! vertical wire
            dist_sq=(ver_x-FGTAR_X(gtar_ir))**2+
     $              (ver_z-FGTAR_Z(gtar_ir))**2
            ori_str='(v)'
         ENDIF
         IF(ma_verblvl.GE.8)
     $        WRITE(*,*) ' magetgtar: Wire:',gtar_ir,
     $                     ori_str,' dist:',SQRT(dist_sq)
         IF(dist_sq.LE.min_dist_sq) THEN
            min_dist_sq=dist_sq
            magetgtar=gtar_ir
         ENDIF
      ENDDO

      IF(magetgtar.EQ.0) THEN
         error=5
         IF(ma_verblvl.GE.2) WRITE(*,*) 
     $        ' magetgtar: Internal Bug encountered.',
     $        'No GTAR row number found)'
         GOTO 999 ! Return
      ELSE
         error=0
      ENDIF

      
c      IF(ma_verblvl.GE.8) THEN
c         IF(ABS(FGTAR_X(magetgtar)).LT.0.2)THEN ! horizontal wire
c            WRITE(*,*) ' magetgtar: horizontal wire'
c         ELSE                   ! vertical wire
c            WRITE(*,*) ' magetgtar: vertical wire'
c         ENDIF
c      ENDIF

      IF(magetgtar.GT.0.AND.magetgtar.LE.NGTAR()) THEN
        IF(ma_verblvl.GT.9) THEN
c         WRITE(*,*) ' magetgtar: GTAR row number=',magetgtar
         WRITE(*,'(a,i3,a,F7.4,'' /'',F7.4,'' /'',F7.4)'),
     $          ' magetgtar: Target (',magetgtar,') x/y/z=',
     $          FGTAR_X(magetgtar),
     $          FGTAR_Y(magetgtar),
     $          FGTAR_Z(magetgtar)
        ENDIF
      ELSE
         error=6
         IF(ma_verblvl.GE.2)
     $        WRITE(*,*) ' magetgtar: GTAR row number=',magetgtar,
     $        '(out of range)'
      ENDIF
      GOTO 999 ! Return

 999  CONTINUE
      IF(ma_verblvl.GE.20)
     $     WRITE(*,*) ' magetgtar: finished'
      RETURN  

      END

************************************************************************
* End of MAGETGTAR
************************************************************************
