      subroutine mamuseg_mtra(imtra,ierror)

      implicit none

************************************************************************
*     Author     : Peter J. Weyers <weyers@ifh.de>
*                  using code by Rainer Mankel 
*     Description: fills RSEG Table rows with information retrieved from the
*                  MC for MTRA row number imtra simulating the muon detector
*                  reconstruction to be used as a seed for the upstream
*                  RANGER algorithm
      INTEGER imtra ! in    MTRA row number
      INTEGER ierror! out   0=success
                    !       1=track skipped because it wasn't a muon
                    !       2=No usable MIMP found
                    !       3=Problems with MMBOOK()
*
************************************************************************

#include "arte/Arte.fi"
#include "auto/MIMP.fi"
#include "auto/MTRA.fi"
#include "auto/MTRX.fi"
#include "auto/RSEG.fi"

#include "match98.fi"
#include "rang/zdomain.fi"
#include "pris/zmpack.fi"
#include "rang/zf.fi"

      integer irseg_new,i,j
      INTEGER imimp,indx,imimp_tmp
      INTEGER old_nrseg
      REAL zmu,x,y,tx,ty,dz,eta,dx(5)
      PARAMETER(zmu = 1600.)
      REAL sx,sy,stx,sty
*      parameter (sx=.5,sy=2.,stx=.002,sty=.027) ! mu resolution
*      parameter (sx=.4,sy=1.4,stx=.007,sty=.018) ! mu resolution
      parameter (sx=.17,sy=.92,stx=.012,sty=.020) ! mu resolution


#include "arte/Arte.ff"
#include "auto/MIMP.ff"
#include "auto/MTRA.ff"
#include "auto/MTRX.ff"
#include "auto/RSEG.ff"


      ierror=100
      IF(OMTRA_TOPP(imtra).NE.code_muplus.AND.
     $   OMTRA_TOPP(imtra).NE.code_muminus) THEN
         ierror=1
         IF(ma_verblvl.GE.10) WRITE(*,*)
     $        ' mamuseg_mtra: MC track ',imtra,' is not a muon',
     $        ' - skipped'
         RETURN
      ENDIF

      IF(ma_verblvl.GE.10)
     $     WRITE(*,*) " mamuseg_mtra: Searchin MIMP for lepton",
     $     imtra
      imimp=0
c     locate last mimp
      indx=0
      DO WHILE(maslcmimp_mtra(imimp_tmp,indx,imtra)) 
         if(fmimp_z(imimp).LT.fmimp_z(imimp_tmp).AND.
     $        fmimp_z(imimp_tmp).LT.1450.)
     $        imimp=imimp_tmp
      ENDDO

      IF(imimp.EQ.0.OR.fmimp_z(imimp).lt.1300.) THEN
         IF(ma_verblvl.GT.4)
     $     WRITE(*,*) ' mamuseg_mtra: no usable MIMP found - skipping!'
         ierror=2
         RETURN
      ELSE 
         IF(ma_verblvl.GT.8) WRITE(*,*) 
     $     ' mamuseg_mtra: found MIMP',imimp,' z=',fmimp_z(imimp)

         dz = zmu - fmimp_z(imimp)
         
         x = fmimp_x(imimp) + dz * fmimp_tx(imimp) ! propagate to mu
         y = fmimp_y(imimp) + dz * fmimp_ty(imimp)
         
         call norran(eta)       ! apply muon system resolution
         x = x + sx * eta
         
         call norran(eta)
         y = y + sy * eta
         
         call norran(eta)
         tx = fmimp_tx(imimp) + stx * eta
         
         call norran(eta)
         ty = fmimp_ty(imimp) + sty * eta
         
         dx(1) = sx
         dx(2) = sy
         dx(3) = stx
         dx(4) = sty
         dx(5) = 10000.
         
c ----   -- Fill RSEG row entries 
         
         old_nrseg=NRSEG()
         CALL MMBOOK(JRSEG,1)      ! increase row counter
         irseg_new = NRSEG()
         IF(irseg_new.GT.old_nrseg) THEN
             
            Q(QRSEG_ZF_L())  = zmu
            Q(QRSEG_XF_L())  = x
            Q(QRSEG_YF_L())  = y 
            Q(QRSEG_TXF_L()) = tx 
            Q(QRSEG_TYF_L()) = ty
            Q(QRSEG_PF_L())  = 0.
            
            Q(QRSEG_TIME_L())=  0 ! ?????????????
            Q(QRSEG_DTIM_L())=  0 ! ?????????????
            
            Q(QRSEG_ZE_L())  =  FRSEG_ZF(irseg_new)
            Q(QRSEG_XE_L())  =  FRSEG_XF(irseg_new)
            Q(QRSEG_YE_L())  =  FRSEG_YF(irseg_new)
            Q(QRSEG_TXE_L()) =  FRSEG_TXF(irseg_new)
            Q(QRSEG_TYE_L()) =  FRSEG_TYF(irseg_new)
            Q(QRSEG_PE_L())  =  FRSEG_PF(irseg_new) 
            
            DO i=1,5
               DO j=1,5
                  IF(i.EQ.j) THEN
                     Q(QRSEG_CVE_L(matrix_pack(i,j)))= dx(i)**2
                     Q(QRSEG_CVF_L(matrix_pack(i,j)))= dx(i)**2
                  ELSE
                     Q(QRSEG_CVE_L(matrix_pack(i,j)))= 0
                     Q(QRSEG_CVF_L(matrix_pack(i,j)))= 0
                  ENDIF
               ENDDO               !
            ENDDO                  !
            
            CALL MMBREL(JRSMT,irseg_new,imtra)
            
            Q(QRSEG_CHI_L()) = 0 ! ? 
            IQ(IQRSEG_FIT_L()) = 0 ! Without Momentum information
            IQ(IQRSEG_CMP_L()) = RSEGc_muon
            ierror=0
         ELSE
            IF(ma_verblvl.GE.3) WRITE(*,*) ' mamuseg_mtra:',
     $           'CALL MMBOOK(JRSEG,1) did not increas NREG()'
            ierror=3
         ENDIF
      ENDIF

      RETURN
      END

