
      SUBROUTINE MAPFIT98(rseg_ir,tar,dtar,iter,error)

************************************************************************
* Author     : Peter J. Weyers <weyers@ifh.de>
* Description: routine to reconstruct the momentum of particles in the
*              HERA-B '98 run, using the RSEG table entries for the track
*              parameters after the magnet and assuming that the particle
*              comes from a known position (target)
*
* Arguments:
*
*  rseg_ir:  INTEGER in   Number of reconstructed Segment
*  tar(3):   REAL    in   Target Koordinates (x,y,z)
*  dtar(2):  REAL    in   Errors in Target Koordinats(dx,dy)
*  iter:     INTEGER in   number of iterations
*
*  error:    INTEGER out  = 0 : success
*                         else: some Error, no ARTE table entries
*                               are changed
*
************************************************************************

      IMPLICIT NONE

      INTEGER rseg_ir
      REAL tar(3),dtar(2)
      INTEGER iter,error

#include "arte/Arte.fi"
#include "auto/RSEG.fi"
#include "match98.fi"

      INTEGER i,j,l,k,count
      INTEGER dinverror,rkerror

      DOUBLE PRECISION prec(4),rkd(5,5),zout,fstep ! for RKMAIN()

      DOUBLE PRECISION cov1(5,5) ! cov. matr. from RSEG table
      DOUBLE PRECISION cov2(5,5) ! cov. matr. after rkutta extrapolation
      DOUBLE PRECISION cov3(5,5) ! cov. matr. after target constraint
      DOUBLE PRECISION CHHC(5,5)
      DOUBLE PRECISION HVH(5,5)
      DOUBLE PRECISION dft(2)
      DOUBLE PRECISION Ht(2,5)
      DOUBLE PRECISION Xf(6)
      DOUBLE PRECISION Xt(6)
      DOUBLE PRECISION Xft(6)
      DOUBLE PRECISION V(2,2)

      INTEGER ibuffer(5)         ! for DINV()

      integer matrix_pack(5,5)   ! for RSEG_CVF acces (copied from  R. M.)
      data matrix_pack
     +     / 1,  2,  4,  7, 11,
     +       2,  3,  5,  8, 12,
     +       4,  5,  6,  9, 13,
     +       7,  8,  9, 10, 14,
     +      11, 12, 13, 14, 15 /
      save matrix_pack

#include "arte/Arte.ff"
#include "auto/RSEG.ff"

      IF(ma_verblvl.GE.20) THEN
         WRITE(*,*) 'mapfit98:',
     $     ' called with rseg_ir=',rseg_ir
         WRITE(*,*) '         ',
     $     '             tar=    ',tar
         WRITE(*,*) '         ',
     $     '             dtar=   ',dtar
         WRITE(*,*) '         ',
     $     '             iter=   ',iter
      ENDIF


c --coordinates after track fit --!
      Xf(1)=FRSEG_ZF(rseg_ir)     !
      Xf(2)=FRSEG_XF(rseg_ir)     !
      Xf(3)=FRSEG_YF(rseg_ir)     !
      Xf(4)=FRSEG_TXF(rseg_ir)    !
      Xf(5)=FRSEG_TYF(rseg_ir)    !
c --------------------------------!

c -target covariance matrix -!
                             !
      V(1,1)=dtar(1)**2      !
      V(1,2)=0               !
      V(2,1)=0               !
      V(2,2)=dtar(2)**2      !
c ---------------------------!

c -variables and precision requirement for rkutta ------!  
      zout=tar(3)                                       !  
                                                        !  
      DO i=1,4                                          !  
         prec(i)   = ma_rkprec(i)/(Xf(1)-zout)          !  
      ENDDO                                             !  
      fstep=ma_rkfstep                                  !  
c ------------------------------------------------------!

c ---- Read the covariance matrix elements from the ARTE table !
                                                               !
      DO i=1,5                                                 !
         DO j=1,5                                              !
            cov1(i,j)=FRSEG_CVF(matrix_pack(i,j),rseg_ir)      !
         ENDDO                                                 !
      ENDDO                                                    !
c -------------------------------------------------------------!

      cov1(5,5)=100000. ! Set momentum error to something "close to infinitiy"

      Xt(6)=0           ! initial q/p

c ---- Begin of iteration loop --------------------------------------------!
                                                                           !
      DO count=1,iter                                                      !
                                                                           !
         Xf(6)=Xt(6)                                                       !
c         WRITE(*,*) 'mapfit98: call rkmain with Xf(6)=',Xf(6)               !
* Argument in/out description                                              !
* ICOVAR   in     = 0    no     calculation of derivative matrix
*                 = 1    normal calculation of derivative matrix
*                 = 2    fast   calculation of derivative matrix
* zout     in     z value for output parameters                            !
* fstep    in     first step size                                          !
* Xf(6)    in     input track parameter (z,x,y,tx,ty,Q/p*sign(cos g))      !
* prec(4)  in     precision for POUT(2-5) / cm in z                        !
* Xft(6)   out    output track parameters                                  !
* rkd(5,5) out    derivatives d POUT(2-6) / d PIN(2-6)                     !
* rkerror  out    = 0 ok                                                   !
*                 = 1 track curls                                          !
*                 = 2 FSTEP = 0                                            !
*                 = 3 rkstep call limit exceeded                           !
c         WRITE(*,*) ' mapfit98: line',__LINE__
         CALL RKMAIN(1,zout,fstep,Xf,prec,Xft,rkd,rkerror)                 !
c         WRITE(*,*) ' mapfit98: line',__LINE__
         IF(rkerror.EQ.3) THEN
            IF(ma_verblvl.GE.6)
     $       WRITE(*,*) 'mapfit98: rkmain() returned error=3 - ignoring' !
            rkerror=0
         ENDIF
                                                                           !  
         IF(rkerror.NE.0) GOTO 800 ! Error handling                        !
                                                                           !
c         WRITE(*,                                                         !
c     &     '(''x/y/z   out ='',F10.4,F10.4,F8.2,'' tan='',F10.6,F10.6)')  !
c     &                 SNGL(Xft(2)),SNGL(Xft(3)),SNGL(Xft(1)),            !
c     &                 Xft(4),                                            !
c     &                 Xft(5)                                             !
c                                                                          !
c         WRITE(*,*)'cov1:'                                                !
c         DO i=1,5                                                         !
c            WRITE(*,                                                      !
c     $                '(G11.4,G11.4,G11.4,G11.4,G11.4)')                  !
c     $                 (cov1(i,k),k=1,5)                                  !
c         ENDDO                                                            !
c                                                                          !
c         WRITE(*,*)'rkd:'                                                 !
c                                                                          !
c         DO i=1,5                                                         !
c            WRITE(*,                                                      !
c     $                '(G11.4,G11.4,G11.4,G11.4,G11.4,G11.4)')            !
c     $           (rkd(i,k),k=1,5)                                         !
c         ENDDO                                                            !
                                                                           !
         DO i=1,2                                                          !
            DO j=1,5                                                       !
               IF(i.EQ.j) THEN                                             !
                  Ht(i,j)=1                                                !
               ELSE                                                        !
                  Ht(i,j)=0                                                !
               ENDIF                                                       !
            ENDDO                                                          !
         ENDDO                                                             !
                                                                           !
         dft(1)=Xft(2)-tar(1)                                              !
         dft(2)=Xft(3)-tar(2)                                              !
                                                                           !
         DO i=1,5                                                          !
            DO k=1,5                                                       !
               COV2(i,k)=0                                                 !
               DO l=1,5                                                    !
                  COV2(i,k)= COV2(i,k)+                                    !
     $                 rkd(i,l)*(COV1(l,1)*rkd(k,1) +                      !
     $                 COV1(l,2)*rkd(k,2) +                                !
     $                 COV1(l,3)*rkd(k,3) +                                !
     $                 COV1(l,4)*rkd(k,4) +                                !
     $                 COV1(l,5)*rkd(k,5))                                 !
               ENDDO                                                       !
            ENDDO                                                          !
         ENDDO                                                             !
                                                                           ! 
c -- Take into account multiple scattering: -----------------------------! !
c (Divide distance by sqrt(2.) to approximate distribution of scattering ! ! 
c  places - this is a guessed value)                                     ! !
                                                                         ! ! 
         IF(ma_pfit_rlen.GT.0) THEN                                      ! !
            IF(ma_verblvl.GE.18) WRITE(*,*) 'mapfit98:',                 ! !
     $           ' Using rlen:',ma_pfit_rlen                             ! ! 
              COV2(1,1)=COV2(1,1)+ (0.021*ABS(Xf(6))*                    ! !
     $                      sqrt(ma_pfit_rlen)*(Xf(1)-zout)/SQRT(2.))**2 ! ! 
                                                                         ! ! 
              COV2(2,2)=COV2(2,2)+ (0.021*ABS(Xf(6))*                    ! ! 
     $                      sqrt(ma_pfit_rlen)*(Xf(1)-zout)/SQRT(2.))**2 ! ! 
                                                                         ! ! 
                                                                         ! ! 
c It seems that the error in the tangents is dominated by the spatial    ! ! 
c error                                                                  ! ! 
              COV2(3,3)=COV2(3,3)+ (0.021*ABS(Xf(6))*                    ! ! 
     $                       sqrt(ma_pfit_rlen))**2                      ! ! 
                                                                         ! ! 
              COV2(4,4)=COV2(4,4)+ (0.021*ABS(Xf(6))*                    ! ! 
     $                       sqrt(ma_pfit_rlen))**2                      ! ! 
           ENDIF                                                         ! ! 
c -----------------------------------------------------------------------! !
                                                                           !
c         WRITE(*,*)'cov2:'                                                !
c         DO i=1,5                                                         !
c            WRITE(*,                                                      !
c     $                '(G11.4,G11.4,G11.4,G11.4,G11.4)')                  !
c     $                 (cov2(i,k),k=1,5)                                  !
c         ENDDO                                                            !
                                                                           !
c  Use two statements instead of loop because of many zero elements --!    !
         HVH(1,1)=Ht(1,1)/V(1,1)*Ht(1,1)                              !    !
         HVH(2,2)=Ht(2,2)/V(2,2)*Ht(2,2)                              !    !
c         DO i=1,5                                                    !    !
c            DO j=1,5                                                 !    !
c                  HVH(i,j) = Ht(1,i)/V(1,1)*Ht(1,j) +                !    !
c     $                       Ht(2,i)/V(2,2)*Ht(2,j)                  !    !
c            ENDDO                                                    !    !
c         ENDDO                                                       !    !
c---------------------------------------------------------------------!    !
         CALL DINV(5,cov2,5,ibuffer,dinverror)                             !
                                                                           !
         IF(dinverror.NE.0) GOTO 700 ! Error handling                      !
                                                                           !
c   Use two statements instead of loop because of many zero elements----!  !
                                                                        !  !
         DO i=1,5                                                       !  !
            DO j=1,5                                                    !  !
               cov3(i,j)=cov2(i,j)                                      !  !
            ENDDO                                                       !  !
         ENDDO                                                          !  !
                                                                        !  !
         cov3(1,1)=cov3(1,1)+HVH(1,1)                                   !  !
         cov3(2,2)=cov3(2,2)+HVH(2,2)                                   !  !
                                                                        !  !
c         DO i=1,5                                                      !  !
c            DO j=1,5                                                   !  !
c               cov3(i,j)=cov2(i,j)+HVH(i,j)                            !  !
c            ENDDO                                                      !  !
c         ENDDO                                                         !  !
c-----------------------------------------------------------------------!  !
         CALL DINV(5,cov3,5,ibuffer,dinverror)                             !
                                                                           !
         IF(dinverror.NE.0) GOTO 700 ! Error handling                      !
                                                                           !
c         WRITE(*,*)'cov3:'                                                !
c                                                                          !
c         DO i=1,5                                                         !
c            WRITE(*,                                                      !
c     $                '(G11.4,G11.4,G11.4,G11.4,G11.4)')                  !
c     $                 (cov3(i,k),k=1,5)                                  !
c         ENDDO                                                            !
                                                                           !
                                                                           !
c        Matrix Multipication, zero values commented out                   !
         DO i=1,5                                                          !
            Xt(i+1)=Xft(i+1)-(                                             !
     $                        COV3(i,1)*Ht(1,1)                            !
c     $                      + COV3(i,2)*Ht(1,2)                           !
c     $                      + COV3(i,3)*Ht(1,3)                           !
c     $                      + COV3(i,4)*Ht(1,4)                           !
c     $                      + COV3(i,5)*Ht(1,5)                           !
     $                                         )/V(1,1)*dft(1)             !
     $                      -(                                             !
c     $                        COV3(i,1)*Ht(2,1) +                         !
     $                        COV3(i,2)*Ht(2,2)                            !
c     $                      + COV3(i,3)*Ht(2,3)                           !
c     $                      + COV3(i,4)*Ht(2,4)                           !
c     $                      + COV3(i,5)*Ht(2,5)                           !
     $                                          )/V(2,2)*dft(2)            !
         ENDDO                                                             !
                                                                           !
         Xt(1)=Xft(1)                                                      !
                                                                           !
c         IF (Xt(6).NE.0.) THEN                                            !
c            WRITE(*,*) 'mapfit98: Estimated p/q (fit): ',1/Xt(6)            !
c         ELSE                                                             !
c            WRITE(*,*) 'mapfit98: Estimated p/q (fit): 1/0.000000'          !
c         ENDIF                                                            !
                                                                           !
c        Fill the track parameters in the Arte table                       !
                                                                           !
         Q(QRSEG_ZF(rseg_ir))   = SNGL(Xt(1))                              !
         Q(QRSEG_XF(rseg_ir))   = SNGL(Xt(2))                              !
         Q(QRSEG_YF(rseg_ir))   = SNGL(Xt(3))                              !
         Q(QRSEG_TXF(rseg_ir))  = SNGL(Xt(4))                              !
         Q(QRSEG_TYF(rseg_ir))  = SNGL(Xt(5))                              !
         Q(QRSEG_PF(rseg_ir))   = SNGL(Xt(6))                              !
         IQ(IQRSEG_CMP(rseg_ir))=IOR(IRSEG_CMP(rseg_ir),RSEGc_tar)         !
c        Fill the new covariance matrix elements in the ARTE table         !
                                                                           !
         DO i=1,5                                                          !
            DO j=1,i                                                       !
               Q(QRSEG_CVF(matrix_pack(i,j),rseg_ir))=SNGL(cov3(i,j))      !
            ENDDO                                                          !
         ENDDO                                                             !
                                                                           !
      ENDDO                                                                !
                                                                           !
c ---- End of iteration loop ----------------------------------------------!

      error=0

      RETURN
c ----------------- Error handling ---------------------------------------!
  700 CONTINUE                                                            !
                                                                          !
      IF(ma_verblvl.GE.4)
     $     WRITE(*,*) 'mapfit98: Singularity in DINV' !
                                                                          !
      error = -1                                                          !
                                                                          !
      GOTO 890                                                            !
c --------------------------------------------------------                !
 800  CONTINUE                                                            !
                                                                          !
      IF(ma_verblvl.GE.4)
     $     WRITE(*,'(A,I2)'),'mapfit98: Error in RKMAIN(), rkerror = ',   !
     $                                               rkerror              !
                                                                          !
      error=rkerror                                                       !
                                                                          !
      GOTO 890                                                            !
c --------------------------------------------------------                !
                                                                          !
 890  CONTINUE                                                            !
                                                                          !
                                                                          !
      RETURN                                                              !
c ------------------------------------------------------------------------!

 900  END

************************************************************************
* End of MAPFIT98
************************************************************************
