
      SUBROUTINE MATXFIT98(rseg_ir,tar,dtar,iter,error)

************************************************************************
*
* Author     : Peter J. Weyers
* Description: routine to reconstruct the tangents of a particle track in the
*              HERA-B '98 run, using the RSEG table entries for the track
*              parameters after the magnet and assuming that the particle
*              comes from a known position (target).
*              It assumes the segment to consist of only one point 
*              and having length 0. The direction information at the end
*              coordinates FRSEG_T?E(irseg_ir) are updated.
*              If IRSEG_CMP(rseg_ir) is 0, it changes the first coordinates
*              of the Segment to that of the target, else only the direction
*              of the first coordinates are updates (like those at the end,
*              as zero length is assumed)
*
* Arguments:
*
*  rseg_ir:  INTEGER in   Number of reconstructed Segment
*  tar(3):   REAL    in   Target Koordinates (x,y,z)
*  dtar(2):  REAL    in   Errors in Target Koordinats(dx,dy)
*  iter:     INTEGER in   number of iterations
*
*  error:    INTEGER out  = 0 : success
*                         else: some Error, no ARTE table entries
*                               are changed
*
************************************************************************

      IMPLICIT NONE

      INTEGER rseg_ir
      REAL tar(3),dtar(2)
      INTEGER iter,error

#include "arte/Arte.fi"
#include "auto/RSEG.fi"
#include "arte/RSEG_cmp.fi"
#include "match98.fi"

      INTEGER i,j,k,l,m,count
      INTEGER dinverror,rkerror

      DOUBLE PRECISION prec(4)   ! precision to be achieved by RKMAIN()
      DOUBLE PRECISION rkd(5,5)  ! deviation matrix given by RKMAIN()
      DOUBLE PRECISION zout      ! z value RKMAIN() is aiming at
      DOUBLE PRECISION fstep     ! for RKMAIN()

      DOUBLE PRECISION cov1(5,5) ! cov. matr. at initial point
      DOUBLE PRECISION cov2(5,5) ! cov. matr. after rkutta extrapolation
      DOUBLE PRECISION cov3(5,5) ! cov. matr. after target constraint
      DOUBLE PRECISION cov3prev(5,5) ! prev. cov. matr. after target constr.
                                     ! (used in case of errors)
      DOUBLE PRECISION cov4(5,5) ! cov. matr. at initial point recalculated

      DOUBLE PRECISION CHHC(5,5)
      DOUBLE PRECISION HVH(5,5)
      DOUBLE PRECISION dft(2)    ! distance between predicted and target x,y
      DOUBLE PRECISION Ht(5,5)
      DOUBLE PRECISION Xf(6)     ! initial track parameters (z,x,y,tx,ty,Q/p)
      DOUBLE PRECISION Xft(6)    ! track parameters given by RKMAIN()
      DOUBLE PRECISION Xt(6)     ! Xft(6) corrected by dft(2)
      DOUBLE PRECISION V(2,2)    ! target covariance matrix
      DOUBLE PRECISION A(5,5)    ! rkd(5,5) inverted (for "smoother")
      DOUBLE PRECISION Xtf(6)    ! recalculated track parameters at first point

      INTEGER ibuffer(5)         ! for DINV()

      DOUBLE PRECISION rkdA(6,6) ! for informal messages
      CHARACTER*4 Xname(6)       ! for informal messages
      data Xname / ' z ',' x ',' y ',' tx',' ty','Q/p' /

      integer matrix_pack(5,5)   ! for RSEG_CVF acces (copied from  R. M.)
      data matrix_pack
     +     / 1,  2,  4,  7, 11,
     +       2,  3,  5,  8, 12,
     +       4,  5,  6,  9, 13,
     +       7,  8,  9, 10, 14,
     +      11, 12, 13, 14, 15 /
      save matrix_pack

#include "arte/Arte.ff"
#include "auto/RSEG.ff"


c --coordinates after track fit --!
      Xf(1)=FRSEG_ZF(rseg_ir)     !
      Xf(2)=FRSEG_XF(rseg_ir)     !
      Xf(3)=FRSEG_YF(rseg_ir)     !
      Xf(4)=FRSEG_TXF(rseg_ir)    !
      Xf(5)=FRSEG_TYF(rseg_ir)    !
      Xf(6)=FRSEG_PF(rseg_ir)     ! initial q/p
c --------------------------------!

c -target covariance matrix -!
                             !
      V(1,1)=dtar(1)**2      !
      V(1,2)=0               !
      V(2,1)=0               !
      V(2,2)=dtar(2)**2      !
c ---------------------------!

c c --- Add Error for numerical uncertainity ---------------------------!
c       V(1,1)=V(1,1)+ (0.021*ABS(Xf(6))*sqrt(0.1)*(Xf(1)-zout))**2     !
c       V(2,2)=V(2,2)+ (0.021*ABS(Xf(6))*sqrt(0.1)*(Xf(1)-zout))**2     !
c C --------------------------------------------------------------------!


      IF(ma_verblvl.GE.20)   WRITE(*,*) ' matxfit98:',
     $     'tar/dtar=',tar(1),tar(2),tar(3),' /',dtar(1),dtar(2)


c -variables and precision requirement for rkutta --------!  
      zout=tar(3)                                         !  
                                                          !  
      DO i=1,4                                            !  
         prec(i)   = ma_rkprec(i)/(Xf(1)-zout)            !  
      ENDDO                                               !  
      fstep=ma_rkfstep                                    !  
c --------------------------------------------------------!

c ---- Read the covariance matrix elements from the ARTE table !
                                                               !
      DO i=1,5                                                 !
         DO j=1,5                                              !
            cov1(i,j)=FRSEG_CVF(matrix_pack(i,j),rseg_ir)      !
         ENDDO                                                 !
      ENDDO                                                    !
c -------------------------------------------------------------!

      Xt(6)=Xf(6)
      Xft(6)=Xf(6)
      dft(1)=0
      dft(2)=0
c ---- Begin of iteration loop --------------------------------------------!
                                                                           !
      DO count=1,iter                                                      !
                                                                           !
c         Xf(6)=Xt(6)                                                      !
c         WRITE(*,*) 'matxfit98: call rkmain with Xf(6)=',Xf(6)       !
                                                                           !
         IF(ma_verblvl.GE.20.AND.(count.EQ.1.OR.count.EQ.iter))
     $                      CALL maprintmx5(cov1,'cov1')                   !
                                                                           !
* Argument in/out description                                              !
* ICOVAR   in     = 0    no     calculation of derivative matrix
*                 = 1    normal calculation of derivative matrix
*                 = 2    fast   calculation of derivative matrix
* zout     in     z value for output parameters                            !
* fstep    in     first step size                                          !
* Xf(6)    in     input track parameter (z,x,y,tx,ty,Q/p*sign(cos g))      !
* prec(4)  in     precision for POUT(2-5) / cm in z                        !
* Xft(6)   out    output track parameters                                  !
* rkd(5,5) out    derivatives d POUT(2-6) / d PIN(2-6)                     !
* rkerror  out    = 0 ok                                                   !
*                 = 1 track curls                                          !
*                 = 2 FSTEP = 0                                            !
*                 = 3 rkstep call limit exceeded                           !
                                                                           !
         CALL RKMAIN(1,zout,fstep,Xf,prec,Xft,rkd,rkerror)                 !
         IF(rkerror.EQ.3) THEN                                             !
            IF(ma_verblvl.GE.7)  WRITE(*,*) ' matxfit98:',            !
     $           'rkmain() returned error=3 - ignoring' !                  !
           rkerror=0                                                       !
         ENDIF                                                             !
                                                                           !  
         IF(rkerror.NE.0) GOTO 800 ! Error handling                        !
                                                                           !

         IF(ma_verblvl.GE.20.AND.(count.EQ.1.OR.count.EQ.iter))
     $                      CALL maprintmx5(rkd,'rkd ')
c -----------------------------------------------------------
         DO i=1,5                                                          !
            DO j=1,5                                                       !
               IF(i.EQ.j.AND.i.LE.2) THEN                                  !
                  Ht(i,j)=1                                                !
               ELSE                                                        !
                  Ht(i,j)=0                                                !
               ENDIF                                                       !
            ENDDO                                                          !
         ENDDO                                                             !
            
         dft(1)=Xft(2)-tar(1)                                              !
         dft(2)=Xft(3)-tar(2)                                              !

         IF(ma_verblvl.GE.10) 
     $        WRITE(*,*) ' matxfit98: dft x/y =',dft
                                                                           !
         DO i=1,5                                                          !
            DO k=1,5                                                       !
               COV2(i,k)=0                                                 !
               DO l=1,5                                                    !
                  COV2(i,k)= COV2(i,k)+                                    !
     $                 rkd(i,l)*(COV1(l,1)*rkd(k,1) +                      !
     $                           COV1(l,2)*rkd(k,2) +                      !
     $                           COV1(l,3)*rkd(k,3) +                      !
     $                           COV1(l,4)*rkd(k,4) +                      !
     $                           COV1(l,5)*rkd(k,5))                       !
               ENDDO                                                       !
            ENDDO                                                          !
         ENDDO                                                             !
                                                                           !
c Take into account multiple scattering:                                   !
c (Divide distance by 1.57 to approximate distribution of scattering       !
c places. This is an empiric value for particles traveling from the target !
c to the ECAL averaging over all angles                                    !
c Theoretical value for equally distributed scattering media               !
c would be sqrt(3)                                                         !
         IF(ma_xfit_rlen.GT.0)                                             !
     $        COV2(1,1)=COV2(1,1)+ (0.021*ABS(Xf(6))*                      !
     $                       sqrt(ma_xfit_rlen)*(Xf(1)-zout)/1.57)**2      !
         IF(ma_yfit_rlen.GT.0)                                             !
     $        COV2(2,2)=COV2(2,2)+ (0.021*ABS(Xf(6))*                      !
     $                       sqrt(ma_yfit_rlen)*(Xf(1)-zout)/1.57)**2      !
                                                                           !
c         WRITE(*,*) 'Special Message to Peter:',Xf(1)-zout                !
                                                                           !
c It seems that the error in the tangents is dominated by the spatial error!
c                                                                          !
         IF(ma_txfit_rlen.GT.0)                                            !
     $        COV2(3,3)=COV2(3,3)+ (0.021*ABS(Xf(6))*                      !
     $                       sqrt(ma_txfit_rlen))**2                       !
         IF(ma_tyfit_rlen.GT.0)                                            !
     $        COV2(4,4)=COV2(4,4)+ (0.021*ABS(Xf(6))*                      !
     $                       sqrt(ma_tyfit_rlen))**2                       !
                                                                           !
                                                                           !
         IF(ma_verblvl.GE.20.AND.(count.EQ.1.OR.count.EQ.iter))            !
     $                      CALL maprintmx5(cov2,'cov2')                   !
                                                                           !
                                                                           !
c  Use two statements instead of loop because of many zero elements --!    !
        HVH(1,1)=Ht(1,1)/V(1,1)*Ht(1,1)                               !    !
        HVH(2,2)=Ht(2,2)/V(2,2)*Ht(2,2)                               !    !
c         DO i=1,5                                                    !    !
c            DO j=1,5                                                 !    !
c                  HVH(i,j) = Ht(1,i)/V(1,1)*Ht(1,j) +                !    !
c     $                       Ht(2,i)/V(2,2)*Ht(2,j)                  !    !
c            ENDDO                                                    !    !
c         ENDDO                                                       !    !
c---------------------------------------------------------------------!    !
                                                                           !
         IF(ma_verblvl.GE.21.AND.(count.EQ.1.OR.count.EQ.iter)) THEN       !
            CALL maprintmx5(Ht,'Ht')                                       !
            CALL maprintmx5(HVH,'HVH')                                     !
         ENDIF                                                             !
                                                                           !
c   Use two statements instead of loop because of many zero elements----!  !
                                                                        !  !
         DO i=1,5                                                       !  !
            DO j=1,5                                                    !  !
               cov3prev(i,j)=cov3(i,j) ! save old values                !  !
               cov3(i,j)=cov2(i,j)                                      !  !
            ENDDO                                                       !  !
         ENDDO                                                          !  !
                                                                        !  !
         IF(ma_verblvl.GE.20.AND.(count.EQ.1.OR.count.EQ.iter))         !  !
     $                      CALL maprintmx5(cov3,'cov3a')               !  !
                                                                        !  !
         CALL DINV(5,cov3,5,ibuffer,dinverror)                          !  !
                                                                        !  !
         IF(ma_verblvl.GE.20.AND.(count.EQ.1.OR.count.EQ.iter))         !  !
     $                      CALL maprintmx5(cov3,'cov3b')               !  !
                                                                        !  !
         IF(dinverror.NE.0) GOTO 700 ! Error handling                   !  !
                                                                        !  !
         cov3(1,1)=cov3(1,1)+HVH(1,1)                                   !  !
         cov3(2,2)=cov3(2,2)+HVH(2,2)                                   !  !
                                                                        !  !
c         DO i=1,5                                                      !  !
c            DO j=1,5                                                   !  !
c               cov3(i,j)=cov3(i,j)+HVH(i,j)                            !  !
c            ENDDO                                                      !  !
c         ENDDO                                                         !  !
         IF(ma_verblvl.GE.20.AND.(count.EQ.1.OR.count.EQ.iter))         !  !
     $                      CALL maprintmx5(cov3,'cov3c')               !  !
                                                                        !  !
c-----------------------------------------------------------------------!  !
         CALL DINV(5,cov3,5,ibuffer,dinverror)                             !
                                                                           !
         IF(dinverror.NE.0) GOTO 700 ! Error handling                   !  !
                                                                        !  !
         IF(ma_verblvl.GE.20.AND.(count.EQ.1.OR.count.EQ.iter))            !
     $                      CALL maprintmx5(cov3,'cov3')                   !
                                                                           !
c Add Factor for multiple scattering                                       !
                                                                           !
c         cov3(3,3)=cov3(3,3)+(0.021/Xt(6)*SQRT(0.001))**2                 !
c         cov3(4,4)=cov3(4,4)+(0.021/Xt(6)*SQRT(0.000001))**2              !
c                                                                          !
c         IF(ma_verblvl.GE.20.AND.(count.EQ.1.OR.count.EQ.iter)) THEN      !
c            WRITE(*,*) "With Mult. Scat. Error:"                          !
c            CALL maprintmx5(cov3,'cov3')                                  !
c         ENDIF                                                            !
                                                                           !
         IF(dinverror.NE.0) THEN                                           ! 
            DO i=1,5            ! recover values of  cov3                  !
               DO j=1,5                                                    !
                  cov3(i,j)=cov3prev(i,j)                                  !
               ENDDO                                                       !
            ENDDO                                                          !
            GOTO 700            ! Error handling                           !
         ENDIF                                                             !
                                                                           !
                                                                           !
c        Matrix Multipication, zero values commented out                   !
         DO i=1,5                                                          !
            Xt(i+1)=Xft(i+1)-(                                             !
     $                        COV3(i,1)*Ht(1,1)                            !
     $                      + COV3(i,2)*Ht(1,2)                           !
     $                      + COV3(i,3)*Ht(1,3)                           !
     $                      + COV3(i,4)*Ht(1,4)                           !
     $                      + COV3(i,5)*Ht(1,5)                           !
     $                                         )/V(1,1)*dft(1)             !
     $                      -(                                             !
     $                        COV3(i,1)*Ht(2,1) +                         !
     $                        COV3(i,2)*Ht(2,2)                            !
     $                      + COV3(i,3)*Ht(2,3)                           !
     $                      + COV3(i,4)*Ht(2,4)                           !
     $                      + COV3(i,5)*Ht(2,5)                           !
     $                                          )/V(2,2)*dft(2)            !
         ENDDO                                                             !
                                                                           !
         Xt(1)=Xft(1)  ! z                                                 !
                                                                           !
         IF(ma_verblvl.GE.20.) WRITE(*,*) ' matxfit98:',              !
     $        'p vorher/nachher',Xf(6),Xt(6) !

c cccccccccccccccccccccccccccccccccccccccccccccccccc                       !
         IF(count.EQ.iter) THEN
c         IF(count.GT.iter/2) THEN
c         IF(.TRUE.) THEN
                                                                           !
            DO i=1,5                                                       !
               DO j=1,5                                                    !
                  A(i,j)=rkd(i,j)                                          !
               ENDDO                                                       !
            ENDDO                                                          !
                                                                           !
            CALL DINV(5,A,5,ibuffer,dinverror)                             !
                                                                           !
            IF(dinverror.NE.0) THEN                                        ! 
               GOTO 700            ! Error handling                        !
            ENDIF                                                          !
                                                                           !
            IF(ma_verblvl.GE.20.AND.(count.EQ.1.OR.count.EQ.iter))         !
     $                         CALL maprintmx5(A,'A ')                     !
                                                                           !
            DO i=1,5                                                       !
                  Xtf(i+1)=Xf(i+1)+ A(i,1)*(-dft(1))                       !
     $                            + A(i,2)*(-dft(2))                       !
c                                 + A(i,3)* 0                              !
c                                 + A(i,4)* 0                              !
c                                 + A(i,5)* 0                              !
            ENDDO                                                          !
            Xtf(1)=Xf(1)  ! z                                              !
                                                                           !
         ELSE                                                              !
            Xtf(1)=Xf(1)                                                   !
            Xtf(2)=Xf(2)                                                   !
            Xtf(3)=Xf(3)                                                   !
c            Xtf(4)=Xf(4)                                                  !
c            Xtf(5)=Xf(5)                                                  !
            Xtf(4)=Xf(4)-(Xft(2)-tar(1))/rkd(1,3)                          !
            Xtf(5)=Xf(5)-(Xft(3)-tar(2))/rkd(2,4)                          !
c            Xtf(4)=Xf(4)+(Xt(2)-tar(1))/(Xf(1)-zout)                      !
c            Xtf(5)=Xf(5)+(Xt(3)-tar(2))/(Xf(1)-zout)                      !
c            Xtf(4)=Xf(4)+(Xft(2)-tar(1))/(Xf(1)-zout)                     !
c            Xtf(5)=Xf(5)+(Xft(3)-tar(2))/(Xf(1)-zout)                     !
            Xtf(6)=Xf(6)                                                   !
            IF(ma_verblvl.GE.20) THEN                                      !
               WRITE(*,*) ' matxfit98: zout-Xf(1):',zout-Xf(1)             !
               WRITE(*,*) '            rkd       :',rkd(1,3),rkd(2,4)      !
            ENDIF                                                          !
         ENDIF                                                             !
c cccccccccccccccccccccccccccccccccccccccccccccccccc                       !
                                                                           !
c         IF(ma_verblvl.GE.20.AND.(count.EQ.1.OR.count.EQ.iter))            !
c     $                      THEN                                           !
c            WRITE(*,*) ' matxfit98: ',                                !
c     $           '             Xf                    Xf-Xtf/Xf'            !
c                                                                           !
c            DO i=1,6                                                       !
c               IF(Xf(i).NE.0) THEN                                         !
c                  WRITE(*,*) '            ',Xname(i),":",Xf(i),            !
c     $                 (Xf(i)-Xtf(i))/Xf(i)                                !
c               ELSE                                                        !
c                  WRITE(*,*) '            ',Xname(i),":",Xf(i),            !
c     $                 (Xf(i)-Xtf(i)),'/0'                                 !
c               ENDIF                                                       !
c            ENDDO                                                          !
c         ENDIF                                                             !
                                                                            !
          DO i=1,6                                                          !
             Xf(i)=Xtf(i)                                                   !
          ENDDO                                                             !

                                                                           !
                                                                           !
c         cov1(3,3)=cov4(3,3)                                               !
c         cov1(4,4)=cov4(4,4)                                               !
                                                                           !
                                                                           !
c ------------------------------------------------------------------------ !
         IF(ma_verblvl.GE.10)  WRITE(*,'(A,A,F8.6,F8.6)')                  !
     $        ' matxfit98:',                                               !
     $        ' Estimated tx/ty (fit): '                                   !
     $        ,Xf(4),Xf(5)                                                 !
                                                                           !
                                                                           !
      ENDDO                                                                !
                                                                           !
c ---- End of iteration loop ----------------------------------------------!

         
      error=0

      GOTO 890

c ----------------- Error handling ---------------------------------------!
  700 CONTINUE                                                            !
                                                                          !
      IF(ma_verblvl.GE.3)                                                 !
     $     WRITE(*,*) 'matxfit98: Singularity in DINV' !                  !
                                                                          !
      error = -1                                                          !
                                                                          !
      GOTO 890                                                            !
c --------------------------------------------------------                !
 800  CONTINUE                                                            !
                                                                          !
      IF(ma_verblvl.GE.3) WRITE(*,'(A,A,I3)'),'matxfit98:',               !
     $     'Error in RKMAIN(), rkerror = ',                               !
     $     rkerror              !                                         !
                                                                          !
      error=rkerror                                                       !
                                                                          !
      GOTO 890                                                            !
c --------------------------------------------------------                !
                                                                          !
                                                                          !
                                                                          !
c ------------------------------------------------------------------------!

 890  CONTINUE                                                            


c      DO i=1,5                                                       
c         DO j=1,5                                                    
c            A(i,j)=rkd(i,j)                                          
c         ENDDO                                                       
c      ENDDO                                                          
c                                                                     
c      CALL DINV(5,A,5,ibuffer,dinverror)                             
c                                                                     
c      IF(dinverror.NE.0) THEN                                        
c         IF(ma_verblvl.GE.3)                                         
c     $        WRITE(*,*) 'matxfit98: Singularity in DINV' 
c                                                                     
c         error = -1                                                  
c      ENDIF                                                          

c Compute Matrix C^T_E = C_E + A_E ( C^T_T - C^E_T)  (A_E)^t         
                                                                     
                                                                     
      DO i=1,5                                                       
         DO j=1,5                                                    
            cov4(i,j)=cov1(i,j)                                      
            DO k=1,5                                                 
               DO l=1,5                                              
                  cov4(i,j)=cov4(i,j)+                               
     $                 A(i,k)*(cov3(k,l)-cov2(k,l))*A(j,l)           
               ENDDO                                                 
            ENDDO                                                    
         ENDDO                                                       
      ENDDO                                                          
                                                                     
c Take into account mult. scat.  errors ---------------------------------- !
                                                                           !
         IF(ma_txfit_rlen.GT.0)                                            !
     $        cov4(3,3)=cov4(3,3)+ (0.021*ABS(Xf(6))*                      !
     $                       sqrt(ma_txfit_rlen))**2                       !
         IF(ma_tyfit_rlen.GT.0)                                            !
     $        cov4(4,4)=cov4(4,4)+ (0.021*ABS(Xf(6))*                      !
     $                       sqrt(ma_tyfit_rlen))**2                       !
c -------------------------------------------------------------------------!

      IF(ma_verblvl.GE.20.AND.(count.EQ.1.OR.count.EQ.iter))         
     $                   CALL maprintmx5(cov4,'cov4')                
c     Fill the track parameters in the Arte table                       

      Q(QRSEG_TXE(rseg_ir)) = Xf(4)
      Q(QRSEG_TYE(rseg_ir)) = Xf(5)

      Q(QRSEG_CVE(matrix_pack(3,3),rseg_ir))=SNGL(cov4(3,3))      
      Q(QRSEG_CVE(matrix_pack(4,4),rseg_ir))=SNGL(cov4(4,4))      

      IF(IRSEG_CMP(rseg_ir).EQ.0) THEN
         IF(ma_verblvl.GE.8) WRITE(*,*) ' matxfit98.F:',
     $        ' Extending Segment to Target'
         Q(QRSEG_ZF(rseg_ir)) = SNGL(Xt(1))                                
         Q(QRSEG_XF(rseg_ir)) = SNGL(Xt(2))                                
         Q(QRSEG_YF(rseg_ir)) = SNGL(Xt(3))                                
         Q(QRSEG_TXF(rseg_ir))= SNGL(Xt(4))                                
         Q(QRSEG_TYF(rseg_ir))= SNGL(Xt(5))                                
         Q(QRSEG_PF(rseg_ir)) = SNGL(Xt(6))                                
         
         IQ(IQRSEG_CMP(rseg_ir)) = IOR(IRSEG_CMP(rseg_ir),RSEGc_tar)

c        Fill the new covariance matrix elements in the ARTE table         
                                                                           
         DO i=1,5                                                          
            DO j=1,i                                                       
               Q(QRSEG_CVF(matrix_pack(i,j),rseg_ir))=SNGL(cov3(i,j))      
            ENDDO                                                          
         ENDDO                                                               

      ELSE
         IF(ma_verblvl.GE.8) WRITE(*,*) ' matxfit98.F:',
     $        ' Updating values for zero-length Segment'


         Q(QRSEG_TXF(rseg_ir)) = Xf(4)                                     
         Q(QRSEG_TYF(rseg_ir)) = Xf(5)                                     

         Q(QRSEG_CVF(matrix_pack(3,3),rseg_ir))=SNGL(cov4(3,3))      
         Q(QRSEG_CVF(matrix_pack(4,4),rseg_ir))=SNGL(cov4(4,4))      

      ENDIF

      RETURN

 900  END

************************************************************************
* End of MATXFIT98
************************************************************************


